@@ -626,6 +626,13 @@ public:
      */
     void select (  );
 
+    bool canHaveSelection();
+    long selectionStart();
+    long selectionEnd();
+    void setSelectionStart(long start);
+    void setSelectionEnd(long start);
+    void setSelectionRange(long start, long end);
+
     /**
      * Simulate a mouse-click. For <code> INPUT </code> elements whose
      * <code> type </code> attribute has one of the following values:
@@ -1161,6 +1168,12 @@ public:
      */
     void focus (  );
 
+    long selectionStart();
+    long selectionEnd();
+    void setSelectionStart(long start);
+    void setSelectionEnd(long start);
+    void setSelectionRange(long start, long end);
+
     /**
      * Select the contents of the <code> TEXTAREA </code> .
      */
