@@ -104,9 +104,11 @@ namespace KJS {
            InputFocus, InputMaxLength, InputDefaultChecked, InputDisabled,
            InputChecked, InputForm, InputAccessKey, InputAlign, InputAlt,
            InputName, InputSrc, InputUseMap, InputSelect, InputClick,
-           TextAreaAccessKey, TextAreaName, TextAreaDefaultValue, TextAreaSelect,
+           InputSelectionStart, InputSelectionEnd, InputSetSelectionRange,
+           TextAreaAccessKey, TextAreaName, TextAreaDefaultValue, TextAreaSelect, TextAreaSetSelectionRange,
            TextAreaCols, TextAreaDisabled, TextAreaForm, TextAreaType,
            TextAreaTabIndex, TextAreaReadOnly, TextAreaRows, TextAreaValue,
+           TextAreaSelectionStart, TextAreaSelectionEnd,
            TextAreaBlur, TextAreaFocus, ButtonBlur, ButtonFocus, ButtonForm, ButtonTabIndex, ButtonName,
            ButtonDisabled, ButtonAccessKey, ButtonType, ButtonValue, LabelHtmlFor,
            LabelForm, LabelAccessKey, FieldSetForm, LegendForm, LegendAccessKey,
