@@ -86,6 +86,8 @@ public:
 
     virtual void parseHTMLAttribute(HTMLAttributeImpl *);
     virtual void attach();
+    void close();
+    virtual void willRemove();
     virtual void detach();
     virtual bool rendererIsNeeded(khtml::RenderStyle *);
     virtual khtml::RenderObject *createRenderer(RenderArena *, khtml::RenderStyle *);
@@ -151,7 +153,6 @@ public:
     int totalRows() const { return m_totalRows; }
     int totalCols() const { return m_totalCols; }
     int border() const { return m_border; }
-    virtual void detach();
 
     virtual void recalcStyle( StyleChange ch );
     
