@@ -647,7 +647,7 @@ enum EUserDrag {
 // CSS3 User Select Values
 
 enum EUserSelect {
-    SELECT_AUTO, SELECT_NONE, SELECT_TEXT
+    SELECT_AUTO, SELECT_NONE, SELECT_TEXT, SELECT_ELEMENT
 };
 
 // Word Break Values. Matches WinIE, rather than CSS3
