@@ -485,32 +485,30 @@ void RenderTable::splitColumn( int pos, int firstSpan )
     columns.resize( oldSize + 1 );
     int oldSpan = columns[pos].span;
 //     qDebug("splitColumn( %d,%d ), oldSize=%d, oldSpan=%d", pos, firstSpan, oldSize, oldSpan );
-    KHTMLAssert( oldSpan > firstSpan );
+    KHTMLAssert(oldSpan > firstSpan);
     columns[pos].span = firstSpan;
-    memmove( columns.data()+pos+1, columns.data()+pos, (oldSize-pos)*sizeof(ColumnStruct) );
+    memmove(columns.data()+pos+1, columns.data()+pos, (oldSize-pos)*sizeof(ColumnStruct));
     columns[pos+1].span = oldSpan - firstSpan;
 
     // change width of all rows.
     RenderObject *child = firstChild();
-    while ( child ) {
-	if ( child->isTableSection() ) {
+    while (child) {
+	if (child->isTableSection()) {
 	    RenderTableSection *section = static_cast<RenderTableSection *>(child);
-	    int size = section->numRows();
-	    int row = 0;
-	    if ( section->cCol > pos )
+	    if (section->cCol > pos)
 		section->cCol++;
-	    while ( row < size ) {
-		section->grid[row].row->resize( oldSize+1 );
+	    int size = section->numRows();
+	    for (int row = 0; row < size; ++row) {
+		section->grid[row].row->resize(oldSize + 1);
 		RenderTableSection::Row &r = *section->grid[row].row;
-		memmove( r.data()+pos+1, r.data()+pos, (oldSize-pos)*sizeof( RenderTableCell * ) );
-// 		qDebug("moving from %d to %d, num=%d", pos, pos+1, (oldSize-pos-1) );
-		r[pos+1] = r[pos] ? (RenderTableCell *)-1 : 0;
-		row++;
+		memmove(r.data() + pos + 1, r.data() + pos, (oldSize - pos) * sizeof(RenderTableSection::CellStruct));
+		r[pos + 1].cell = 0;
+                r[pos + 1].inColSpan = r[pos].inColSpan || r[pos].cell;
 	    }
 	}
 	child = child->nextSibling();
     }
-    columnPos.resize( numEffCols()+1 );
+    columnPos.resize(numEffCols() + 1);
     setNeedsLayoutAndMinMaxRecalc();
 }
 
@@ -518,29 +516,27 @@ void RenderTable::appendColumn( int span )
 {
     // easy case.
     int pos = columns.size();
-//     qDebug("appendColumn( %d ), size=%d", span, pos );
     int newSize = pos + 1;
-    columns.resize( newSize );
+    columns.resize(newSize);
     columns[pos].span = span;
-    //qDebug("appending column at %d, span %d", pos,  span );
 
     // change width of all rows.
     RenderObject *child = firstChild();
-    while ( child ) {
-	if ( child->isTableSection() ) {
+    while (child) {
+	if (child->isTableSection()) {
 	    RenderTableSection *section = static_cast<RenderTableSection *>(child);
 	    int size = section->numRows();
-	    int row = 0;
-	    while ( row < size ) {
-		section->grid[row].row->resize( newSize );
-		section->cellAt( row, pos ) = 0;
-		row++;
+	    for (int row = 0; row < size; ++row) {
+		section->grid[row].row->resize(newSize);
+                RenderTableSection::CellStruct& c = section->cellAt(row, pos);
+		c.cell = 0;
+		c.inColSpan = false;
 	    }
 
 	}
 	child = child->nextSibling();
     }
-    columnPos.resize( numEffCols()+1 );
+    columnPos.resize(numEffCols() + 1);
     setNeedsLayoutAndMinMaxRecalc();
 }
 
@@ -699,13 +695,13 @@ RenderTableCell* RenderTable::cellAbove(const RenderTableCell* cell) const
     // Look up the cell in the section's grid, which requires effective col index
     if (section && rAbove >= 0) {
         int effCol = colToEffCol(cell->col());
-        RenderTableCell* aboveCell;
+        RenderTableSection::CellStruct aboveCell;
         // If we hit a span back up to a real cell.
         do {
             aboveCell = section->cellAt(rAbove, effCol);
             effCol--;
-        } while (aboveCell == (RenderTableCell *)-1 && effCol >=0);
-        return (aboveCell == (RenderTableCell *)-1) ? 0 : aboveCell;
+        } while (!aboveCell.cell && aboveCell.inColSpan && effCol >=0);
+        return aboveCell.cell;
     } else {
         return 0;
     }
@@ -738,13 +734,13 @@ RenderTableCell* RenderTable::cellBelow(const RenderTableCell* cell) const
     // Look up the cell in the section's grid, which requires effective col index
     if (section && rBelow >= 0) {
         int effCol = colToEffCol(cell->col());
-        RenderTableCell* belowCell;
+        RenderTableSection::CellStruct belowCell;
         // If we hit a colspan back up to a real cell.
         do {
             belowCell = section->cellAt(rBelow, effCol);
             effCol--;
-        } while (belowCell == (RenderTableCell *)-1 && effCol >=0);
-        return (belowCell == (RenderTableCell *)-1) ? 0 : belowCell;
+        } while (!belowCell.cell && belowCell.inColSpan && effCol >=0);
+        return belowCell.cell;
     } else {
         return 0;
     }    
@@ -758,12 +754,12 @@ RenderTableCell* RenderTable::cellLeft(const RenderTableCell* cell) const
         return 0;
     
     // If we hit a colspan back up to a real cell.
-    RenderTableCell* prevCell;
+    RenderTableSection::CellStruct prevCell;
     do {
         prevCell = section->cellAt(cell->row(), effCol-1);
         effCol--;
-    } while (prevCell == (RenderTableCell *)-1 && effCol >=0);
-    return (prevCell == (RenderTableCell *)-1) ? 0 : prevCell;
+    } while (!prevCell.cell && prevCell.inColSpan && effCol >=0);
+    return prevCell.cell;
 }
 
 RenderTableCell* RenderTable::cellRight(const RenderTableCell* cell) const
@@ -771,8 +767,7 @@ RenderTableCell* RenderTable::cellRight(const RenderTableCell* cell) const
     int effCol = colToEffCol(cell->col()+cell->colSpan());
     if (effCol >= numEffCols())
         return 0;
-    RenderTableCell* result = cell->section()->cellAt(cell->row(), effCol);
-    return (result == (RenderTableCell*)-1) ? 0 : result;
+    return cell->section()->cellAt(cell->row(), effCol).cell;
 }
 
 RenderBlock* RenderTable::firstLineBlock() const
@@ -910,9 +905,12 @@ void RenderTableSection::ensureRows(int numRows)
 
         gridRows = numRows;
         int nCols = table()->numEffCols();
+        CellStruct emptyCellStruct;
+        emptyCellStruct.cell = 0;
+        emptyCellStruct.inColSpan = false;
 	for (int r = nRows; r < numRows; r++ ) {
 	    grid[r].row = new Row(nCols);
-	    grid[r].row->fill(0);
+	    grid[r].row->fill(emptyCellStruct);
 	    grid[r].baseLine = 0;
 	    grid[r].height = Length();
 	}
@@ -933,31 +931,9 @@ void RenderTableSection::addCell( RenderTableCell *cell )
     // <TR><TD>1 <TD rowspan="2">2 <TD>3 <TD>4
     // <TR><TD colspan="2">5
     // </TABLE>
-#if 0
-    // find empty space for the cell
-    bool found = false;
-    while ( !found ) {
-	found = true;
-	while ( cCol < nCols && cellAt( cRow, cCol ) )
-	    cCol++;
-	int pos = cCol;
-	int span = 0;
-	while ( pos < nCols && span < cSpan ) {
-	    if ( cellAt( cRow, pos ) ) {
-		found = false;
-		cCol = pos;
-		break;
-	    }
-	    span += columns[pos].span;
-	    pos++;
-	}
-    }
-#else
-    while ( cCol < nCols && cellAt( cRow, cCol ) )
-	cCol++;
-#endif
 
-//       qDebug("adding cell at %d/%d span=(%d/%d)",  cRow, cCol, rSpan, cSpan );
+    while (cCol < nCols && (cellAt(cRow, cCol).cell || cellAt(cRow, cCol).inColSpan))
+	cCol++;
 
     if ( rSpan == 1 ) {
 	// we ignore height settings on rowspan cells
@@ -994,32 +970,35 @@ void RenderTableSection::addCell( RenderTableCell *cell )
 
     int col = cCol;
     // tell the cell where it is
-    RenderTableCell *set = cell;
-    while ( cSpan ) {
+    CellStruct currentCell;
+    currentCell.cell = cell;
+    currentCell.inColSpan = false;
+    while (cSpan) {
 	int currentSpan;
-	if ( cCol >= nCols ) {
-	    table()->appendColumn( cSpan );
+	if (cCol >= nCols) {
+	    table()->appendColumn(cSpan);
 	    currentSpan = cSpan;
 	} else {
-	    if ( cSpan < columns[cCol].span )
-		table()->splitColumn( cCol, cSpan );
+	    if (cSpan < columns[cCol].span)
+		table()->splitColumn(cCol, cSpan);
 	    currentSpan = columns[cCol].span;
 	}
-	int r = 0;
-	while ( r < rSpan ) {
-	    if ( !cellAt( cRow + r, cCol ) ) {
-// 		qDebug("    adding cell at %d, %d",  cRow + r, cCol );
-		cellAt( cRow + r, cCol ) = set;
-	    }
-	    r++;
+
+	for (int r = 0; r < rSpan; r++) {
+            CellStruct& c = cellAt(cRow + r, cCol);
+            if (currentCell.cell && !c.cell)
+                c.cell = currentCell.cell;
+            if (currentCell.inColSpan)
+                c.inColSpan = true;
 	}
 	cCol++;
 	cSpan -= currentSpan;
-	set = (RenderTableCell *)-1;
+        currentCell.cell = 0;
+	currentCell.inColSpan = true;
     }
-    if ( cell ) {
-	cell->setRow( cRow );
-	cell->setCol( table()->effColToCol( col ) );
+    if (cell) {
+	cell->setRow(cRow);
+	cell->setCol(table()->effColToCol(col));
     }
 }
 
@@ -1037,13 +1016,14 @@ void RenderTableSection::setCellWidths()
 	Row &row = *grid[i].row;
 	int cols = row.size();
 	for ( int j = 0; j < cols; j++ ) {
-	    RenderTableCell *cell = row[j];
-// 	    qDebug("cell[%d,%d] = %p", i, j, cell );
-	    if ( !cell || cell == (RenderTableCell *)-1 )
+	    CellStruct current = row[j];
+            RenderTableCell *cell = current.cell;
+
+	    if (!cell)
 		continue;
 	    int endCol = j;
 	    int cspan = cell->colSpan();
-	    while ( cspan && endCol < cols ) {
+	    while (cspan && endCol < cols) {
 		cspan -= table()->columns[endCol].span;
 		endCol++;
 	    }
@@ -1088,14 +1068,15 @@ void RenderTableSection::calcRowHeight()
 	int totalCols = row->size();
 	int totalRows = gridRows;
 
-	for ( int c = 0; c < totalCols; c++ ) {
-	    cell = cellAt(r, c);
-	    if ( !cell || cell == (RenderTableCell *)-1 )
+	for (int c = 0; c < totalCols; c++) {
+	    CellStruct current = cellAt(r, c);
+            cell = current.cell;
+	    if (!cell || current.inColSpan)
 		continue;
-	    if ( r < totalRows - 1 && cellAt(r+1, c) == cell )
+	    if (r < totalRows - 1 && cellAt(r + 1, c).cell == cell)
 		continue;
 
-	    if ( ( indx = r - cell->rowSpan() + 1 ) < 0 )
+	    if ((indx = r - cell->rowSpan() + 1) < 0)
 		indx = 0;
 
             if (cell->overrideSize() != -1) {
@@ -1226,19 +1207,21 @@ int RenderTableSection::layoutRows( int toAdd )
     int leftOffset = hspacing;
 
     int nEffCols = table()->numEffCols();
-    for ( int r = 0; r < totalRows; r++ )
+    for (int r = 0; r < totalRows; r++)
     {
 	Row *row = grid[r].row;
 	int totalCols = row->size();
-        for ( int c = 0; c < nEffCols; c++ )
+        for (int c = 0; c < nEffCols; c++)
         {
-            RenderTableCell *cell = cellAt(r, c);
-            if (!cell || cell == (RenderTableCell *)-1 )
+            CellStruct current = cellAt(r, c);
+            RenderTableCell* cell = current.cell;
+            
+            if (!cell)
                 continue;
-            if ( r < totalRows - 1 && cell == cellAt(r+1, c) )
+            if (r < totalRows - 1 && cell == cellAt(r + 1, c).cell)
 		continue;
 
-            if ( ( rindx = r-cell->rowSpan()+1 ) < 0 )
+            if ((rindx = r-cell->rowSpan() + 1) < 0)
                 rindx = 0;
 
             rHeight = rowPos[r+1] - rowPos[rindx] - vspacing;
@@ -1367,37 +1350,39 @@ void RenderTableSection::paint(PaintInfo& i, int tx, int ty)
     }
     for ( ; endrow > 0; endrow-- ) {
 	if ( ty + rowPos[endrow-1] <= y + h + os)
-	    break;
+            break;
     }
     unsigned int startcol = 0;
     unsigned int endcol = totalCols;
     if ( style()->direction() == LTR ) {
 	for ( ; startcol < totalCols; startcol++ ) {
 	    if ( tx + table()->columnPos[startcol+1] >= x - os)
-		break;
+                break;
 	}
 	for ( ; endcol > 0; endcol-- ) {
 	    if ( tx + table()->columnPos[endcol-1] <= x + w + os)
 		break;
 	}
     }
-    
-    if ( startcol < endcol ) {
+
+    if (startcol < endcol) {
 	// draw the cells
 	for ( unsigned int r = startrow; r < endrow; r++ ) {
 	    unsigned int c = startcol;
 	    // since a cell can be -1 (indicating a colspan) we might have to search backwards to include it
-	    while ( c && cellAt( r, c ) == (RenderTableCell *)-1 )
+	    while (c && cellAt(r, c).inColSpan)
 		c--;
 	    for ( ; c < endcol; c++ ) {
-		RenderTableCell *cell = cellAt(r, c);
-		if (!cell || cell == (RenderTableCell *)-1 )
+                CellStruct current = cellAt(r, c);
+                RenderTableCell *cell = current.cell;
+		
+                if (!cell || (cell->layer() && i.phase != PaintActionCollapsedTableBorders)) 
 		    continue;
                 
                 // Cells must always paint in the order in which they appear taking into account
                 // their upper left originating row/column.  For cells with rowspans, avoid repainting
                 // if we've already seen the cell.
-		if (r > startrow && (cellAt(r-1, c) == cell))
+		if (r > startrow && (cellAt(r-1, c).cell == cell))
 		    continue;
 
 #ifdef TABLE_PRINT
@@ -1451,13 +1436,13 @@ RenderObject* RenderTableSection::removeChildNode(RenderObject* child)
 void RenderTableSection::dump(QTextStream *stream, QString ind) const
 {
     *stream << endl << ind << "grid=(" << grid.size() << "," << table()->numEffCols() << ")" << endl << ind;
-    for ( unsigned int r = 0; r < grid.size(); r++ ) {
-	for ( int c = 0; c < table()->numEffCols(); c++ ) {
-	    if ( cellAt( r,  c ) && cellAt( r, c ) != (RenderTableCell *)-1 )
-		*stream << "(" << cellAt( r, c )->row() << "," << cellAt( r, c )->col() << ","
-			<< cellAt(r, c)->rowSpan() << "," << cellAt(r, c)->colSpan() << ") ";
+    for (unsigned int r = 0; r < grid.size(); r++) {
+	for (int c = 0; c < table()->numEffCols(); c++) {
+	    if (cellAt( r, c).cell && !cellAt(r, c).inColSpan)
+		*stream << "(" << cellAt(r, c).cell->row() << "," << cellAt(r, c).cell->col() << ","
+			<< cellAt(r, c).cell->rowSpan() << "," << cellAt(r, c).cell->colSpan() << ") ";
 	    else
-		*stream << cellAt( r, c ) << "null cell ";
+		*stream << cellAt(r, c).cell << "null cell ";
 	}
 	*stream << endl << ind;
     }
@@ -1702,9 +1687,7 @@ void RenderTableCell::setStyle( RenderStyle *style )
 }
 
 bool RenderTableCell::requiresLayer() {
-    // FIXME: This is only here until we figure out how to position
-    // table cells properly when they have layers.
-    return false;
+    return isPositioned() || style()->opacity() < 1.0f || hasOverflowClip();
 }
 
 // The following rules apply for resolving conflicts and figuring out which border
@@ -2039,7 +2022,7 @@ void RenderTableCell::paint(PaintInfo& i, int _tx, int _ty)
     int os = 2*maximalOutlineSize(i.phase);
     if ((_ty >= i.r.y() + i.r.height() + os) || (_ty + _topExtra + m_height + _bottomExtra <= i.r.y() - os))
         return;
-    
+
     if (i.phase == PaintActionCollapsedTableBorders && style()->visibility() == VISIBLE) {
         int w = width();
         int h = height() + borderTopExtra() + borderBottomExtra();
@@ -2253,8 +2236,19 @@ void RenderTableCell::paintBoxDecorations(PaintInfo& i, int _tx, int _ty)
     int end = kMin(i.r.y() + i.r.height(), _ty + h);
     int mh = end - my;
 
-    if (bgLayer->hasImage() || c.isValid())
-	paintBackground(i.p, c, bgLayer, my, mh, _tx, _ty, w, h);
+    if (bgLayer->hasImage() || c.isValid()) {
+	// We have to clip here because the backround would paint
+        // on top of the borders otherwise.
+        if (m_layer && tableElt->collapseBorders()) {
+            QRect clipRect(_tx + borderLeft(), _ty + borderTop(), w - borderLeft() - borderRight(), h - borderTop() - borderBottom());
+            clipRect = i.p->xForm(clipRect);
+            i.p->save();
+            i.p->addClip(clipRect);
+        }
+        paintBackground(i.p, c, bgLayer, my, mh, _tx, _ty, w, h);
+        if (m_layer && tableElt->collapseBorders())
+            i.p->restore();
+    }
 
     if (style()->hasBorder() && !tableElt->collapseBorders())
         paintBorder(i.p, _tx, _ty, w, h, style());
