@@ -196,6 +196,7 @@ static const char * const eventNames[EventImpl::numEventIds] = {
     "DOMAttrModified",
     "DOMCharacterDataModified",
     "load",
+    "beforeunload",
     "unload",
     "abort",
     "error",
@@ -295,6 +296,15 @@ bool EventImpl::isWheelEvent() const
     return false;
 }
 
+bool EventImpl::storesResultAsString() const
+{
+    return false;
+}
+
+void EventImpl::storeResult(const DOMString&)
+{
+}
+
 // -----------------------------------------------------------------------------
 
 UIEventImpl::UIEventImpl()
@@ -788,3 +798,20 @@ bool RegisteredEventListener::operator==(const RegisteredEventListener &other)
 ClipboardImpl::~ClipboardImpl()
 {
 }
+
+// -----------------------------------------------------------------------------
+
+BeforeUnloadEventImpl::BeforeUnloadEventImpl()
+    : EventImpl(BEFOREUNLOAD_EVENT, false, false)
+{
+}
+
+bool BeforeUnloadEventImpl::storesResultAsString() const
+{
+    return true;
+}
+
+void BeforeUnloadEventImpl::storeResult(const DOMString& s)
+{
+    m_result = s.implementation();
+}
