@@ -87,6 +87,7 @@ public:
         DOMCHARACTERDATAMODIFIED_EVENT,
 	// HTML events
 	LOAD_EVENT,
+	BEFOREUNLOAD_EVENT,
 	UNLOAD_EVENT,
 	ABORT_EVENT,
 	ERROR_EVENT,
@@ -167,6 +168,9 @@ public:
     void setDefaultPrevented(bool defaultPrevented) { m_defaultPrevented = defaultPrevented; }
     bool getCancelBubble() const { return m_cancelBubble; }
 
+    virtual bool storesResultAsString() const;
+    virtual void storeResult(const DOMString&);
+
 protected:
     DOMStringImpl *m_type;
     bool m_canBubble;
@@ -477,6 +481,20 @@ public:
     virtual void setDragImageElement(const Node &, const QPoint &) = 0;
 };
 
+class BeforeUnloadEventImpl : public EventImpl
+{
+public:
+    BeforeUnloadEventImpl();
+
+    virtual bool storesResultAsString() const;
+    virtual void storeResult(const DOMString&);
+
+    DOMString result() const { return m_result.get(); }
+
+private:
+    khtml::SharedPtr<DOMStringImpl> m_result;
+};
+
 } // namespace
 
 #endif
