@@ -242,6 +242,8 @@ public:
     bool hasClass() const   { return m_hasClass; }
     bool hasStyle() const   { return m_hasStyle; }
     bool active() const     { return m_active; }
+    bool inActiveChain() const { return m_inActiveChain; }
+    bool hovered() const { return m_hovered; }
     bool focused() const { return m_focused; }
     bool attached() const   { return m_attached; }
     bool changed() const    { return m_changed; }
@@ -258,6 +260,8 @@ public:
     void setInDocument(bool b=true) { m_inDocument = b; }
     virtual void setFocus(bool b=true) { m_focused = b; }
     virtual void setActive(bool b=true) { m_active = b; }
+    void setInActiveChain(bool b=true) { m_inActiveChain = b; }
+    virtual void setHovered(bool b=true) { m_hovered = b; }
     void setChanged(bool b=true);
 
     unsigned short tabIndex() const { return m_tabIndex; }
@@ -392,6 +396,7 @@ public:
      */
     virtual void detach();
 
+    virtual void willRemove();
     void createRendererIfNeeded();
     virtual khtml::RenderStyle *styleForRenderer(khtml::RenderObject *parent);
     virtual bool rendererIsNeeded(khtml::RenderStyle *);
@@ -492,6 +497,8 @@ protected:
     bool m_specified : 1; // used in AttrImpl. Accessor functions there
     bool m_focused : 1;
     bool m_active : 1;
+    bool m_hovered : 1;
+    bool m_inActiveChain : 1;
     bool m_styleElement : 1; // contains stylesheet text
     bool m_implicit : 1; // implicitely generated by the parser
 
@@ -515,6 +522,8 @@ public:
     virtual bool hasChildNodes (  ) const;
 
     // Other methods (not part of DOM)
+    void willRemove();
+    int willRemoveChild(NodeImpl *child);
     void removeChildren();
     void cloneChildNodes(NodeImpl *clone);
 
@@ -533,6 +542,7 @@ public:
 
     virtual void setFocus(bool=true);
     virtual void setActive(bool=true);
+    virtual void setHovered(bool=true);
     virtual unsigned long childNodeCount() const;
     virtual NodeImpl *childNode(unsigned long index);
 
