@@ -25,6 +25,7 @@
 
 #include "KWQString.h"
 #include "dom_atomicstring.h"
+#include "misc/main_thread_malloc.h"
 
 #ifdef __OBJC__
 @class NSString;
@@ -40,6 +41,8 @@ public:
     KWQFontFamily(const KWQFontFamily &);    
     KWQFontFamily &operator=(const KWQFontFamily &);
         
+    MAIN_THREAD_ALLOCATED;
+
     void setFamily(const DOM::AtomicString &);
     const DOM::AtomicString& family() const { return _family; }
     bool familyIsEmpty() const { return _family.isEmpty(); }
