@@ -28,6 +28,7 @@
 #include <qmap.h>
 
 #include "kjs_window.h"
+#include "dom_docimpl.h"
 
 class KURL;
 
@@ -42,16 +43,20 @@ namespace KJS {
 @interface KWQPageState : NSObject
 {
     DOM::DocumentImpl *document;
+    DOM::NodeImpl *mousePressNode;
     KURL *URL;
     KJS::SavedProperties *windowProperties;
     KJS::SavedProperties *locationProperties;
     KJS::SavedBuiltins *interpreterBuiltins;
     QMap<int, KJS::ScheduledAction*> *pausedActions;
+    DOM::DocumentImpl::ParseMode parseMode;
 }
 
 - initWithDocument:(DOM::DocumentImpl *)doc URL:(const KURL &)u windowProperties:(KJS::SavedProperties *)wp locationProperties:(KJS::SavedProperties *)lp interpreterBuiltins:(KJS::SavedBuiltins *)ib;
 
 - (DOM::DocumentImpl *)document;
+- (DOM::DocumentImpl::ParseMode)parseMode;
+- (DOM::NodeImpl *)mousePressNode;
 - (KURL *)URL;
 - (KJS::SavedProperties *)windowProperties;
 - (KJS::SavedProperties *)locationProperties;
