@@ -109,7 +109,6 @@ public:
 
     const QPalette& palette() const;
     virtual void setPalette(const QPalette &);
-    void unsetPalette();
     
     QStyle &style() const;
     void setStyle(QStyle *);
@@ -123,7 +122,6 @@ public:
     QCursor cursor();
     void unsetCursor();
     bool event(QEvent *);
-    bool hasMouseTracking() const;
 
     void show();
     void hide();
@@ -157,6 +155,14 @@ public:
     
     void setIsSelected(bool isSelected);
 
+    static void beforeMouseDown(NSView *);
+    static void afterMouseDown(NSView *);
+
+    void addToSuperview(NSView *superview);
+    void removeFromSuperview();
+
+    static void setDeferFirstResponderChanges(bool);
+
 private:
     KWQWidgetPrivate *data;
 };
